// Copyright 2017 The Johns Hopkins University Applied Physics Laboratory.
// Licensed under the MIT License. See LICENSE.txt in the project root for full license information.
/***/
/*
Calculates the squared value of error as the L2-distance between the input mesh and the Lidar mesh data. 
Output error is presented in a 2D grid representing the X-Y plane.
Outputs:
- errorRMSL2: squared value error of Z axis error
- valid: flag to indicate Z error whether it is larger or smaller than average + lambda*std

Inputs:
- indexpoint: buffer containing the vertex to cell grid correspondences
- Xref: buffer containing the 2D gridded X values of LIDAR data
- Yref: buffer containing the 2D gridded Y values of LIDAR data
- Zref: buffer containing the 2D gridded Z values of LIDAR data
- X2: buffer containing the 2D gridded X values of input solution
- Y2: buffer containing the 2D gridded Y values of input solution
- Z2: buffer containing the 2D gridded Z values of input solution
- lambda:specific scale of standard deviation for error truncation according to formula: average + lambda*std

Version history: 2017-03-28	Marc Bosch	Initial version
*/
/***/
#include <math.h>
#include "mex.h"
#include <stdio.h>
#include <stdlib.h>
#include <limits>


//double mexbuffer2gridC(double *img, double *curve, double *out, int rows, int cols, int Niter);//region level set + re-init + ignoring curve regularization force
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{	
    if(nlhs != 2){
        mexErrMsgTxt("Function has 2 return value");
        return;
    }

    if(nrhs !=8){
        mexErrMsgTxt("Usage: [errorRMSL2,valid]=mexErrorMetricsC(double *indexpoint, double *Xref, double *Yref, double *Zref, double *X2, double *Y2, double *Z2, dobule lambda)");
        return;
    }
    
    /* get the image buffer*/    
    double *indexpoint = (double*) mxGetPr(prhs[0]);
	double *Xref = (double*) mxGetPr(prhs[1]);
	double *Yref = (double*) mxGetPr(prhs[2]);
	double *Zref = (double*) mxGetPr(prhs[3]);
	double *X2 = (double*) mxGetPr(prhs[4]);
	double *Y2 = (double*) mxGetPr(prhs[5]);    
	double *Z2 = (double*) mxGetPr(prhs[6]);
	double* plambda = mxGetPr(prhs[7]);
 	double lambda = *plambda;
	int* dims = (int*) mxGetDimensions(prhs[0]);
    int len = MAX(dims[0],dims[1]);
	//allocate outputs
	plhs[0] = mxCreateDoubleMatrix(dims[0],dims[1], mxREAL);
    double *errorRMS = mxGetPr(plhs[0]);  
    for (int i=0; i < len; i++)
        errorRMS[i] = 0;//dims[0];  
	plhs[1] = mxCreateDoubleMatrix(dims[0],dims[1], mxREAL);
    double *validRMS = mxGetPr(plhs[1]);  
    for (int i=0; i < len; i++)
        validRMS[i] = 0;//dims[0]; 
	double mean_RMS=0;
	double std_RMS=0;
	for (int i=0;i<len;i++)
	{
		/*errorRMS[i] = (Xref[(int)indexpoint[i]-1]-X2[i])*
			(Xref[(int)indexpoint[i]-1]-X2[i])+
			(Yref[(int)indexpoint[i]-1]-Y2[i])*
			(Yref[(int)indexpoint[i]-1]-Y2[i])+
			(Zref[(int)indexpoint[i]-1]-Z2[i])*
			(Zref[(int)indexpoint[i]-1]-Z2[i]);*/
		errorRMS[i] = (Zref[(int)indexpoint[i]-1]-Z2[i])*
			(Zref[(int)indexpoint[i]-1]-Z2[i]);
		mean_RMS+=errorRMS[i];
		std_RMS+=(errorRMS[i]*errorRMS[i]);
	}
	len = (len<2) ? 2 : len;
	double temp=mean_RMS;
    mean_RMS = mean_RMS/len;
    std_RMS =  sqrt((std_RMS-temp*mean_RMS)/(len-1));
	

	for (int i=0;i<len;i++)
	{
		validRMS[i]=1;
		if (errorRMS[i]>mean_RMS+lambda*std_RMS)
			validRMS[i]=0;
	}   
   
   
}
